@@ -1,6 +1,8 @@
+from datetime import datetime
 from http import HTTPStatus
 import json
-from typing import Any, Dict, List
+from ssl import SSLContext
+from typing import Any, Dict, List, Type
 
 import aiohttp
 from graphql import OperationType
@@ -9,15 +11,29 @@ from rath.operation import GraphQLException, GraphQLResult, Operation
 from rath.links.base import AsyncTerminatingLink
 from rath.links.errors import AuthenticationError
 import logging
+import certifi
+import ssl
 
 logger = logging.getLogger(__name__)
 
 
+class DateTimeEncoder(json.JSONEncoder):
+    def default(self, o):
+        if isinstance(o, datetime):
+            return o.isoformat()
+
+        return json.JSONEncoder.default(self, o)
+
+
 class AIOHttpLink(AsyncTerminatingLink):
     endpoint_url: str
+    ssl_context: SSLContext = Field(
+        default_factory=lambda: ssl.create_default_context(cafile=certifi.where())
+    )
     auth_errors: List[HTTPStatus] = Field(
         default_factory=lambda: (HTTPStatus.FORBIDDEN,)
     )
+    json_encoder: Type[json.JSONEncoder] = Field(default=DateTimeEncoder, exclude=True)
 
     _session = None
 
@@ -46,7 +62,7 @@ class AIOHttpLink(AsyncTerminatingLink):
             # Enumerate the file streams
             # Will generate something like {'0': <_io.BufferedReader ...>}
             file_streams = {str(i): files[path] for i, path in enumerate(files)}
-            operations_str = json.dumps(payload)
+            operations_str = json.dumps(payload, cls=self.json_encoder)
 
             data.add_field(
                 "operations", operations_str, content_type="application/json"
@@ -67,7 +83,10 @@ class AIOHttpLink(AsyncTerminatingLink):
             payload["variables"] = operation.variables
             post_kwargs = {"json": payload}
 
-        async with aiohttp.ClientSession() as session:
+        async with aiohttp.ClientSession(
+            connector=aiohttp.TCPConnector(ssl=self.ssl_context),
+            json_serialize=lambda x: json.dumps(x, cls=self.json_encoder),
+        ) as session:
             async with session.post(
                 self.endpoint_url, headers=operation.context.headers, **post_kwargs
             ) as response:
