@@ -12,6 +12,8 @@ FILE_CLASSES = (
     aiohttp.StreamReader,
     AsyncGenerator,
 )
+
+
 from typing import Any, Dict, Tuple, Type
 
 
@@ -46,6 +48,11 @@ def parse_variables(
             files[path] = obj
             return None
 
+        elif hasattr(obj, "__file__"):
+            # extract obj from its parent and put it into files instead.
+            files[path] = obj.value
+            return None
+
         else:
             # base case: pass through unchanged
             return obj
@@ -56,30 +63,10 @@ def parse_variables(
 
 
 class FileExtraction(ContinuationLink):
-    async def aquery(self, operation: Operation) -> Operation:
-        operation.variables, operation.context.files = parse_variables(
-            operation.variables
-        )
-        return await self.next.aquery(operation)
-
-    async def asubscribe(self, operation: Operation) -> Operation:
+    async def aexecute(self, operation: Operation) -> Operation:
         operation.variables, operation.context.files = parse_variables(
             operation.variables
         )
 
-        async for result in self.next.asubscribe(operation):
-            yield result
-
-    def query(self, operation: Operation) -> Operation:
-        operation.variables, operation.context.files = parse_variables(
-            operation.variables
-        )
-        return self.next.query(operation)
-
-    def subscribe(self, operation: Operation) -> Operation:
-        operation.variables, operation.context.files = parse_variables(
-            operation.variables
-        )
-
-        for result in self.next.subscribe(operation):
+        async for result in self.next.aexecute(operation):
             yield result
