@@ -3,11 +3,11 @@ from typing import AsyncIterator, Awaitable, Callable, Dict, Optional
 
 from pydantic import Field, validator
 from rath.links.base import AsyncTerminatingLink
+from rath.links.testing.mock import AsyncMockResolver
 from rath.operation import GraphQLResult, Operation
 from graphql import FieldNode, OperationType
 import uuid
 from rath.links.errors import TerminatingLinkError
-import abc
 
 
 def target_from_node(node: FieldNode) -> str:
@@ -24,23 +24,6 @@ def target_from_node(node: FieldNode) -> str:
     )
 
 
-class AsyncMockResolver(abc.ABC):
-    """Async Mock Resolver
-
-    Mimiks to a resolver that can be used in a GraphQL StatefulMockLink.
-    You need to subclass this and implement the resolve_* methods.
-
-
-    """
-
-    def __getitem__(self, key):
-        return getattr(self, f"resolve_{key}")
-
-    def __contains__(self, key):
-        return hasattr(self, f"resolve_{key}")
-        #
-
-
 class ConfigurationError(TerminatingLinkError):
     """A Configuration Error"""
 
@@ -56,6 +39,8 @@ class AsyncStatefulMockLink(AsyncTerminatingLink):
 
     """
 
+    timeout: float = 2
+
     query_resolver: Dict[str, Callable[[Operation], Awaitable[Dict]]] = Field(
         default_factory=dict, exclude=True
     )
@@ -87,21 +72,25 @@ class AsyncStatefulMockLink(AsyncTerminatingLink):
             return v.to_dict()
         return v
 
-    async def __aenter__(self) -> None:
-        self._connected = True
+    async def aconnect(self):
         self._futures = {}
         self._inqueue = asyncio.Queue()
         self._connection_task = asyncio.create_task(self.resolving())
+        self._connected = True
 
-    async def __aexit__(self, *args, **kwargs) -> None:
-        self._connected = False
+    async def adisconnect(self):
         self._connection_task.cancel()
+        self._connected = False
 
         try:
             await self._connection_task
         except asyncio.CancelledError:
             pass
 
+    async def __aexit__(self, *args, **kwargs) -> None:
+        if self._connection_task:
+            await self.adisconnect()
+
     async def resolving(self):
         """A coroutine that resolves the incoming operations in
         an inifite Loop
@@ -145,6 +134,7 @@ class AsyncStatefulMockLink(AsyncTerminatingLink):
                                 f"Mocked Resolver for Query '{op.name.value}' not in resolvers: {self.mutation_resolver}, {self.resolver}  for AsyncMockLink"
                             )
                 resolved = await asyncio.gather(*resolve_futures)
+
                 self._futures[id].set_result(
                     GraphQLResult(
                         data={
@@ -159,7 +149,6 @@ class AsyncStatefulMockLink(AsyncTerminatingLink):
                 raise ConfigurationError(f"No resolver for operation {op}") from t
 
             except Exception as e:
-
                 self._futures[id].set_exception(e)
 
             self._inqueue.task_done()
@@ -167,13 +156,19 @@ class AsyncStatefulMockLink(AsyncTerminatingLink):
     async def submit(self, o, id):
         await self._inqueue.put((o, id))
 
-    async def aquery(self, operation: Operation) -> GraphQLResult:
-        uniqueid = uuid.uuid4()
-        self._futures[uniqueid] = asyncio.Future()
-        await self.submit(operation, uniqueid)
-        return await self._futures[uniqueid]
+    async def aexecute(self, operation: Operation) -> AsyncIterator[GraphQLResult]:
+        if not self._connected:
+            raise TerminatingLinkError("Not connected")
+
+        if (
+            operation.node.operation == OperationType.QUERY
+            or operation.node.operation == OperationType.MUTATION
+        ):
+            uniqueid = uuid.uuid4()
+            self._futures[uniqueid] = asyncio.Future()
+            await self.submit(operation, uniqueid)
+            yield await asyncio.wait_for(self._futures[uniqueid], timeout=self.timeout)
 
-    async def asubscribe(self, operation: Operation) -> AsyncIterator[GraphQLResult]:
         if operation.node.operation == OperationType.SUBSCRIPTION:
             assert (
                 len(operation.node.selection_set.selections) == 1
